SystemOrganization addCategory: #'LightweightClasses-Core'!

!Behavior methodsFor: '*LightweightClasses' stamp: 'hfm 11/9/2008 21:31'!
isLightweightClass

	^ false! !

Behavior subclass: #LightweightClass
	instanceVariableNames: 'name'
	classVariableNames: 'ClassMethod ClassPoolMethod'
	poolDictionaries: ''
	category: 'LightweightClasses-Core'!

!LightweightClass commentStamp: 'hfm 11/16/2008 20:32' prior: 0!
| aDate |

aDate := Date today.
aDate becomeLightweight.
aDate dispatchingClass compile: 'day ^ 48' notifying: nil.
aDate day.

 | aDate |

aDate := Date today.
aDate becomeLightweight.
aDate dispatchingClass compile: 'day ^ 45' notifying: nil.
aDate class!

!LightweightClass class methodsFor: 'instance creation' stamp: 'hfm 11/9/2008 20:02'!
newWithSuper: aClass

	^ self basicNew initializeWithSuper: aClass! !

!LightweightClass class methodsFor: 'instance creation' stamp: 'hfm 11/10/2008 01:40'!
reset
	" self reset "

	ClassMethod := nil.
	ClassPoolMethod := nil.! !

!LightweightClass methodsFor: 'initialize' stamp: 'hfm 11/16/2008 19:32'!
initializeClassMethod
	" Private - Compile the class message into receiver
	so further class requests are redirected appropiately "

	| source |
	
	source := 'class
	self break.
	^ self dispatchingClass superclass'.
	ClassMethod := (( Object compilerClass new
					compile: source 
					in: self
					notifying: nil 
					ifFail: [] ) generateWith: #(0 0 0 0) using: UnstoredMethod) copyWithTrailerBytes: #(0 0 0 0).
	ClassMethod sourceCode: source.! !

!LightweightClass methodsFor: 'initialize' stamp: 'hfm 11/16/2008 19:33'!
initializeClassPoolMethod
	" Private - Squeak Inspector and ObjectExplorer do not
	work without #classPool ? "
	
	| source |
	
	source := 'classPool
		self break.
	^ self dispatchingClass classPool'.
	ClassPoolMethod := (( Object compilerClass new
					compile: source 
					in: self
					notifying: nil 
					ifFail: [] ) generateWith: #(0 0 0 0) using: UnstoredMethod) copyWithTrailerBytes: #(0 0 0 0).
	ClassPoolMethod sourceCode: source.! !

!LightweightClass methodsFor: 'initialize' stamp: 'hfm 11/16/2008 19:33'!
initializeWithSuper: aClass

	| mDict |
	
	self name: ( String streamContents: [ : str | str nextPut: ${; nextPutAll: aClass name; nextPut: $} ] ).
	self superclass: aClass.
	format := aClass format.
	mDict := MethodDictionary new.
	ClassMethod 
		ifNil: [ self initializeClassMethod.
				self initializeClassPoolMethod ].
	mDict at: #class putNoBecome: ( ClassMethod clone methodClass: aClass ).
	mDict at: #classPool putNoBecome: ( ClassPoolMethod clone methodClass: aClass ).
	" Possibly not necessary "
	ClassMethod flushCache.
	ClassPoolMethod flushCache.
	self methodDictionary: mDict.	! !

!LightweightClass methodsFor: 'testing' stamp: 'hfm 11/9/2008 21:31'!
isLightweightClass

	^ true! !

!LightweightClass methodsFor: 'testing' stamp: 'hfm 11/9/2008 21:32'!
isMeta

	^ false! !

!LightweightClass methodsFor: 'accessing' stamp: 'hfm 11/9/2008 20:02'!
name
	^ name! !

!LightweightClass methodsFor: 'accessing' stamp: 'hfm 11/9/2008 20:02'!
name: anObject
	name := anObject! !

!LightweightClass methodsFor: 'printing' stamp: 'hfm 11/9/2008 20:01'!
printOn: aStream
	" See super implementor comment "

	aStream nextPutAll: self name! !

!Object methodsFor: '*LightweightClasses' stamp: 'hfm 11/9/2008 20:12'!
becomeLightweight

	| lightweightClass |
	
	( self dispatchingClass isKindOf: LightweightClass )
		ifFalse: [ lightweightClass := LightweightClass newWithSuper: self class.
				self primitiveChangeClassTo: lightweightClass basicNew ]! !

!Object methodsFor: '*LightweightClasses' stamp: 'hfm 11/9/2008 20:08'!
dispatchingClass
	"Primitive. Answer the object which is the receiver's class. Essential. See 
	Object documentation whatIsAPrimitive."

	<primitive: 111>
	self primitiveFailed! !

Object subclass: #UnstoredMethodInstVars
	instanceVariableNames: 'sourceCode classPool'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LightweightClasses-Core'!

!UnstoredMethodInstVars methodsFor: 'accessing' stamp: 'hfm 11/10/2008 01:29'!
classPool
	"Answer the dictionary of class variables."

	classPool == nil
		ifTrue: [^ Dictionary new ]
		ifFalse: [ ^ classPool]! !

!UnstoredMethodInstVars methodsFor: 'accessing' stamp: 'hfm 11/9/2008 20:57'!
sourceCode
	^ sourceCode! !

!UnstoredMethodInstVars methodsFor: 'accessing' stamp: 'hfm 11/9/2008 20:57'!
sourceCode: anObject
	sourceCode := anObject! !

CompiledMethod variableByteSubclass: #UnstoredMethod
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LightweightClasses-Core'!

UnstoredMethod class
	instanceVariableNames: 'compiledMethodInstVars'!

UnstoredMethod class
	instanceVariableNames: 'compiledMethodInstVars'!

!UnstoredMethod class methodsFor: 'accessing' stamp: 'hfm 11/9/2008 21:02'!
compiledMethodInstVars
	^ compiledMethodInstVars! !

!UnstoredMethod class methodsFor: 'accessing' stamp: 'hfm 11/9/2008 21:02'!
compiledMethodInstVars: anObject
	compiledMethodInstVars := anObject! !

!UnstoredMethod class methodsFor: 'private' stamp: 'hfm 11/9/2008 21:04'!
countAllInstances
	"doIt: [self countAllInstances]"

	| totalCount |
	totalCount := 0.
	Smalltalk garbageCollect.
	Transcript cr.
	self withAllSubclassesDo: [:each |
		| count |
		count := each allInstances size.
		totalCount := totalCount + count.
		Transcript cr; show: each name, ' -> ', count printString].
	Transcript cr; show: '=== TOTAL COUNT -> ', totalCount printString.! !

!UnstoredMethod class methodsFor: 'initialize' stamp: 'hfm 11/9/2008 23:11'!
initialize
	"doIt: [ self initialize ]"

	"--------------------------------------------------------------------------------"
	self tweakClassFormat.
	"--------------------------------------------------------------------------------"
	self
		compiledMethodInstVars: WeakIdentityKeyDictionary new.! !

!UnstoredMethod class methodsFor: 'private' stamp: 'hfm 11/9/2008 23:11'!
tweakClassFormat

	self setFormat: (self format bitOr: 16r200).! !

!UnstoredMethod methodsFor: 'derived accessing' stamp: 'hfm 11/10/2008 01:34'!
classPool
	"Answer the dictionary of class variables."

	| instVars |
	
	self break.
	instVars := self compiledMethodInstVars at: self ifAbsent: [nil].
	
	^ instVars 
		ifNotNil: [ instVars classPool ]! !

!UnstoredMethod methodsFor: 'derived accessing' stamp: 'hfm 11/9/2008 21:06'!
compiledMethodInstVars

	^ self class compiledMethodInstVars! !

!UnstoredMethod methodsFor: 'initialize' stamp: 'hfm 11/9/2008 22:48'!
copyWithTrailerBytes: bytes

	| start end copy |
	start := self initialPC.
	end := self endPC.
	copy := self class newMethod: end - start + 1 + bytes size header: self header.
	1 to: self numLiterals do: [:i | copy literalAt: i put: (self literalAt: i)].
	start to: end do: [:i | copy at: i put: (self at: i)].
	1 to: bytes size do: [:i | copy at: end + i put: (bytes at: i)].
	^ copy! !

!UnstoredMethod methodsFor: 'derived accessing' stamp: 'hfm 11/9/2008 21:03'!
getSource
	
	^ self sourceCode! !

!UnstoredMethod methodsFor: 'private' stamp: 'hfm 11/16/2008 20:30'!
objectAt: anIndex 
	"Lie about what's in our literal frame. This has the bad side-effect that we can't
	view the correct information in inspectors, but the good news is that we stop
	infinite recursion. Good thing the VM doesn't use the basicAt: method :)."

	| object |
	
	object := self realObjectAt: anIndex.
	" self needsWork. "
	^ object isVariableBinding
		ifTrue: [ object value superclass ] 
		ifFalse: [ object ]! !

!UnstoredMethod methodsFor: 'private' stamp: 'hfm 11/16/2008 16:55'!
realObjectAt: anIndex

	^ super objectAt: anIndex! !

!UnstoredMethod methodsFor: 'derived accessing' stamp: 'hfm 11/9/2008 20:59'!
sourceCode

	| instVars |
	instVars := self compiledMethodInstVars at: self ifAbsent: [nil].
	^ instVars ifNotNil: [ instVars sourceCode ]! !

!UnstoredMethod methodsFor: 'derived accessing' stamp: 'hfm 11/9/2008 21:18'!
sourceCode: aString

	( self compiledMethodInstVars 
			at: self 
			ifAbsentPut: [ UnstoredMethodInstVars new ] ) sourceCode: aString.
! !
UnstoredMethod initialize!
